{=====================================================================}
{                                                                     }
{ Registry ComboBox for Windows 95 ver 1.0.2 28/Jun/1997              }
{ http://www.kobira.co.jp/sakura/windows.htm                          }
{ author Seiji-Yasuda seiji@kobira.co.jp                              }
{                                                                     }
{ You may use it freely at your own risk in any kind of environment.  }
{                                                                     }
{=====================================================================}

{$DEFINE USLIB} { for non-Japanese Delphi }

unit RegCBox;

interface

uses
  Windows, SysUtils, Classes, Controls, StdCtrls;


type

  TRegistryComboBoxListType = (rcbltKeyList, rcbltValueList);
  TRegistryComboBoxKey = (
    KEY_CLASSES_ROOT, KEY_CURRENT_USER, KEY_LOCAL_MACHINE,
    KEY_USERS, KEY_CURRENT_CONFIG);

  TCustomRegistryComboBox = class(TCustomComboBox)
  private
    FListType: TRegistryComboBoxListType;
    FKey: TRegistryComboBoxKey;
    FPath: string;
    fParams : TStringList;  // registry value list.
    procedure SetListType(Value: TRegistryComboBoxListType);
    procedure SetKey(Value: TRegistryComboBoxKey);
    procedure SetPath(Value: string);
  protected
    procedure ReadItems;
  public
    constructor Create(owner: TComponent); override;
    destructor Destroy; override;
    procedure Loaded; override;
    procedure ReadList(the_params: TStrings);
  // possible published:
    property ListType: TRegistryComboBoxListType
      read FListType write SetListType;
    property Key: TRegistryComboBoxKey read FKey write SetKey;
    property Path: string read FPath write SetPath;
  end;

  TRegistryComboBox = class(TCustomRegistryComboBox)
  public
    property Text;
  published
//  property Style; {Must be published before Items}
    property Color;
    property Ctl3D;
//  property DragMode;
//  property DragCursor;
//  property DropDownCount;
    property Enabled;
    property Font;
(*$IFNDEF USLIB*)
//  property ImeMode;
(*$ENDIF  USLIB*)
    property ItemHeight;
//  property Items;
//  property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
//  property PopupMenu;
    property ShowHint;
//  property Sorted;
    property TabOrder;
    property TabStop;
//  property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
//  property OnDragDrop;
//  property OnDragOver;
    property OnDrawItem;
//  property OnDropDown;
//  property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
//  property OnMeasureItem;
//  property OnStartDrag;
    property ListType: TRegistryComboBoxListType
      read FListType write SetListType default rcbltKeyList;
    property Key: TRegistryComboBoxKey read FKey write SetKey;
    property Path: string read FPath write SetPath;
  end;


procedure Register;


implementation


procedure TCustomRegistryComboBox.ReadList(the_params: TStrings);
const
  HKeys: array [TRegistryComboBoxKey] of HKEY =
    (HKEY_CLASSES_ROOT, HKEY_CURRENT_USER, HKEY_LOCAL_MACHINE,
     HKEY_USERS, HKEY_CURRENT_CONFIG);
var
  ans : longint; hd : HKEY; Len, i: DWORD;
  tmp : array[0..255] of Char;
  t : TFileTime;
begin
  the_params.Clear;
  // get registory handle.
  if RegOpenKeyExA(
       HKeys[Key],
       StrPCopy(tmp, Path),
       0,
       KEY_ENUMERATE_SUB_KEYS or KEY_QUERY_VALUE,
       hd) <> ERROR_SUCCESS then
    exit;
  // get subkey name and add to the_params.
  if ListType = (rcbltKeyList) then
  begin
    for i := 0 to MaxInt do
    begin
      Len := SizeOf(tmp);
      if (RegEnumKeyExA(hd, i, @tmp, Len, NIL, NIL, NIL, @t) =
          ERROR_SUCCESS) then
        the_params.Add(tmp)
      else
        break;
    end;
  end
  else
  begin
    for i := 0 to MaxInt do
    begin
      Len := SizeOf(tmp);
      if (RegEnumValue(hd, i, @tmp, Len, nil, nil, nil, nil) =
          ERROR_SUCCESS) then
        the_params.Add(tmp)
      else
        break;
    end;
  end;
  // free registory handle.
  RegCloseKey( hd );
end;


//----------------------------------------------------
// Create Items.
procedure TCustomRegistryComboBox.ReadItems;
begin
  if (Parent = nil) then exit;
  ReadList(fParams);
  items.Clear;
  items.AddStrings(fParams);
  if fParams.Count > 0 then begin
     Text := fParams.Strings[0];
     ItemIndex := 0;
  end else begin
     Text := '';
     ItemIndex := -1;
  end;
end;

//----------------------------------------------------
// Called after component got loaded.
procedure TCustomRegistryComboBox.Loaded;
begin
  inherited Loaded;
  ReadItems;
end;

//----------------------------------------------------
// Set properties and rebuild list after:

procedure TCustomRegistryComboBox.SetListType(
  Value: TRegistryComboBoxListType);
begin
  if (Value <> fListType) then
  begin
    fListType := Value;
    if not (csLoading in ComponentState) then
      ReadItems;
  end;
end;

procedure TCustomRegistryComboBox.SetKey(Value: TRegistryComboBoxKey);
begin
  if (Value <> fKey) then
  begin
    fKey := Value;
    if not (csLoading in ComponentState) then
      ReadItems;
  end;
end;

procedure TCustomRegistryComboBox.SetPath(Value: string);
begin
  if (Value <> fPath) then
  begin
    fPath := Value;
    if not (csLoading in ComponentState) then
      ReadItems;
  end;
end;

//----------------------------------------------------
// create
constructor TCustomRegistryComboBox.Create(owner: TComponent);
begin
  inherited Create(owner);
  fParams := TStringList.Create; // make buffer.
  style := csOwnerDrawFixed;
  fListType := rcbltKeyList;
(*$IFNDEF USLIB*)
  ImeMode := imClose;     // for Japanese Kanji.
(*$ENDIF  USLIB*)
end;

//----------------------------------------------------
// destroy
destructor TCustomRegistryComboBox.Destroy;
begin
  fParams.Free;
  inherited Destroy;
end;

//----------------------------------------------------
// register
procedure Register;
begin
  RegisterComponents('samples', [TRegistryComboBox]);
end;

end.
